#!/usr/bin/perl
# 
# sb::Plugin::CommentPatch - Plugin for Serene Bach
# == written by T.Otani <takuya.otani@gmail.com> ===
# == Copyright (C) 2004 SimpleBoxes/SerendipityNZ ==

package sb::Plugin::CommentPatch;
# ==================================================
# // initialization for plugin
# ==================================================
use sb::Plugin ();
sb::Plugin->register_plugin(
	'lang' => {
		'ja' => 'euc',
		'en' => 'ascii',
	},
	'text' => {
		'type'    => 'patch',
		'name'    => 'patch for receiving comment',
		'text'    => 'fixing a bug for receiving comment',
		'author'  => 'takkyun',
		'detail'  => 'http://serenebach.net/',
		'version' => '0.02',
	},
	'file' => undef,
	'data' => undef,
);
use sb::Receipt ();
my $original_func = \&sb::Receipt::_message;
&_initialize_plugin(); # initialize to override default function
sub _initialize_plugin
{
	local $^W = 0; # ignore warning
	no strict 'refs';
	my $weblog = sb::Data->load('Weblog','id'=>0);
	my $flag = undef;
	if ($weblog)
	{
		my $plugin = $weblog->plugin;
		my $name = __FILE__;
		$name =~ s/.*[:\/\\](.*)/$1/;
		$flag = ( index($plugin,'[' . $name . ']') > -1 );
	}
	if ($flag)
	{
		*sb::Receipt::_message = \&_message;
	}
}
sub COOKIE_CHARCODE    (){ 'utf8' };
sub COOKIE_EXPIRES     (){ 8544 }; # 8544 = 365 days
sub _message
{
	my $self  = shift;
	my $entry = sb::Data->load('Entry','id'=>$self->{'id'});
	my $now   = $self->{'time'};
	my $cgi   = $self->{'cgi'};
	my $lang  = sb::Language->get;
	return($entry,"error_no_entry\n") if (!$entry);
	my $code = ($cgi->value('charset') eq '') # charcode not charset to use sb::Language
	         ? $lang->charcode 
	         : $lang->code_for_charset($cgi->value('charset'));
	$lang->checkcode($cgi->value('description'),$code);
	my %com = ( # buffering data for received comment
		'eid'  => $entry->id,
		'stat' => 1,
		'date' => $now,
		'tz'   => sb::Config->get->value('conf_timezone'),
		'body' => &sb::Receipt::_convert_message_text('description',$cgi,$lang),
		'auth' => &sb::Receipt::_convert_message_text('name',$cgi,$lang),
		'mail' => $cgi->value('email'),
		'url'  => $cgi->value('url'),
		'host' => $cgi->value('_addr'),
		'agnt' => $cgi->value('_agnt'),
		'icon' => $cgi->value('icon'),
	);
	$com{'icon'} = int($com{'icon'}) if ($com{'icon'} ne '');
	my $num  = $entry->com;
	my @objs = &sb::Receipt::_load_objs('Message',$entry->id);
	my $spam = $self->_check_spam($entry,\%com,$cgi->value('_refe'),'com');
	eval { # error check
		die("error_no_comment\n") if ( $com{'body'} eq '' );
		die("error_banned\n")     if ( $entry->acm == 0 );
		die("error_banned\n")     if ( !&sb::Receipt::_check_charset($cgi->value('charset')) );
		die("error_no_entry\n")   if ( $entry->stat == 0 );
		die("error_banned\n")     if ( !&sb::Receipt::_check_ip($cgi->value('_addr')) );
		die("error_doubled\n")    if ( &sb::Receipt::_check_redundancy_for_com(\%com,@objs) );
		die("error_banned\n")     if ( sb::Config->get->value('conf_spamstat') and $spam == 0 );
	};
	return($entry,$@) if ($@);
	my $new = sb::Data->add('Message',%com);
	$new->stat($spam);
	$new->stat(0) if ($entry->acm == 2);
	$entry->com( ++$num ) if ($new->stat == 1);
	$self->_sendmail('com',$new,$entry);
	sb::Data->update($new);
	if ($cgi->value('set_cookie'))
	{ # set cookies
		my $conf = sb::Config->get;
		my %data = ();
		$lang->checkcode('',$code) if ($code ne COOKIE_CHARCODE);
		foreach my $key ( @{$conf->value('basic_cookie')} )
		{
			my $value = ($key ne 'checkid') ? $cgi->value($key) : $conf->value('conf_spamid');
			$value = $lang->convert($value,COOKIE_CHARCODE) if ($code ne COOKIE_CHARCODE);
			$data{$key} = $value;
		}
		$cgi->set_cookie(
			'time'   => $now,
			'name'   => $conf->value('basic_cooktag'),
			'expire' => COOKIE_EXPIRES,
			'data'   => \%data,
		);
	}
	return ($new->stat == 1) ? ($entry,'') : ($entry,"error_wait_msg\n");
}
1;
__END__
